(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12484,        370]*)
(*NotebookOutlinePosition[     13148,        393]*)
(*  CellTagsIndexPosition[     13104,        389]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  StyleBox["Methoden der Bioinformatik",
    FontSize->24,
    FontWeight->"Bold"],
  StyleBox[" \n",
    FontSize->12],
  StyleBox["Eine Einf\[UDoubleDot]hrung",
    FontSize->18,
    FontWeight->"Bold"],
  StyleBox["\n",
    FontSize->12],
  StyleBox["Marc-Thorsten H\[UDoubleDot]tt und Manuel Dehnert",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox["\n",
    FontSize->12],
  StyleBox["Springer-Verlag 2006 (ISBN 3-540-25687-3)",
    FontSize->12,
    FontWeight->"Bold"],
  StyleBox["\n",
    FontSize->12]
}], "Subsubtitle"],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "-Exkurs: ",
  "Bereitstellung der Substitutionsmatrix"
}], "Subtitle",
  Background->RGBColor[0.0901961, 0.462745, 0.396078]],

Cell[TextData[{
  "Im Verlauf dieses Kapitels haben wir eine weit \[UDoubleDot]ber die \
Bioinformatik im engeren Sinne hinausreichende Analysemethode kennen gelernt, \
n\[ADoubleDot]mlich die \[CapitalUDoubleDot]bersetzung paarweiser \
\[CapitalADoubleDot]hnlichkeiten von Elementen in einen Clusterbaum. In der \
Bioinformatik findet man solche Clusteralgorithmen vor allem bei einer \
Interpretation von Sequenz \[ADoubleDot]hnlichkeiten (oder auch allgemeiner: \
von Merkmals\[ADoubleDot]hnlichkeiten) in einem evolution\[ADoubleDot]ren \
Sinne. Clusteralgorithmen findet man aber auch bei der Analyse von \
Microarray-Daten, als rein statistisches Werkzeug zum Auffinden von \
Strukturen in numerischen Daten und - wie wir in Kapitel 5 sehen werden - bei \
der Suche nach Modulen (also funktionell oder topologisch abgrenzbaren \
Untereinheiten) in biologischen Netzwerken.\n\nHier visualisieren wir \
beispielhaft ein Kernelement der UPGMA-Methode durch eine explizite, \
schrittweise Implementierung in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". \n\nWir beginnen mit der Distanzmatrix, die wir bereits im \
vorangegangenen Zahlenbeispiel kennen gelernt haben."
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[BoxData[
    \(\(distMatrix = {{0, 1.3, 0.75}, {1.3, 0, 0.34}, {0.75, 0.34, 
            0}};\)\)], "Input"],

Cell[TextData[{
  "\[CapitalUDoubleDot]ber den Formatierungsbefehl ",
  StyleBox["MatrixForm",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  " k\[ODoubleDot]nnen wir die Distanzmatrix ausgeben:"
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[CellGroupData[{

Cell[BoxData[
    \(distMatrix // MatrixForm\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "1.3`", "0.75`"},
            {"1.3`", "0", "0.34`"},
            {"0.75`", "0.34`", "0"}
            },
          RowSpacings->1,
          ColumnSpacings->1,
          ColumnAlignments->{Left}], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Gem\[ADoubleDot]\[SZ] der Interpretation als Distanzmatrix haben wir jetzt \
also eine sehr klare Aufgabe vor uns. Es liegen drei Taxa (zum Beispiel \
Spezies, die wir mit unserer Analyse in einen evolution\[ADoubleDot]ren \
Zusammenhang stellen wollen) vor, zusammen mit ihren paarweisen Abst\
\[ADoubleDot]nden (in Bezug auf irgendeine Observable).\n\nZuerst m\
\[UDoubleDot]ssen wir die kleinste Distanz bestimmen. Unsere vom Alignment \
gewohnte Konstruktion ",
  StyleBox["Position[matrix, Min[matrix]] ",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  "schl\[ADoubleDot]gt hier aufgrund der Nullen auf der Hauptdiagonalen fehl. \
Wir verwenden eine kleine Funktion ",
  StyleBox["makeDiag[n]",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  ", die eine Diagonalmatrix der Gr\[ODoubleDot]\[SZ]e ",
  StyleBox["n\[Times]n",
    FontSlant->"Italic"],
  " mit einer gro\[SZ]en Zahl als Diagonalelemente (und Nullen sonst) \
erzeugt. Diese Matrix ",
  StyleBox["diag",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  " f\[UDoubleDot]gen wir als Zwischenschritt in die Minimumssuche ein. Man \
hat dann die Konstruktion ",
  StyleBox["Position[matrix+diag, Min[matrix+diag]]",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  ". Diese Position, zusammen mit dem Wert des Minimums, schreiben wir in die \
Variable ",
  StyleBox["merge1",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  "."
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[BoxData[
    \(makeDiag[n_] := 
      Table[If[i \[Equal] j, 100, 0], {i, 1, n}, {j, 1, n}]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(diag = makeDiag[Length[distMatrix]]\)], "Input"],

Cell[BoxData[
    \({{100, 0, 0}, {0, 100, 0}, {0, 0, 100}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(merge1 = {\(Position[distMatrix + diag, 
            Min[distMatrix + diag]]\)[\([1]\)], 
        Min[distMatrix + diag]}\)], "Input"],

Cell[BoxData[
    \({{2, 3}, 0.34`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Dieses Wertepaar enth\[ADoubleDot]lt also die beiden Taxa 2 und 3 (erster \
Eintrag), zusammen mit ihrem Abstand (zweiter Eintrag). Wir fassen diese \
Arbeitsschritte in einer Funktion ",
  StyleBox["makeMerge",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  " zusammen."
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[BoxData[
    \(makeMerge[
        matr_] := {\(Position[matr + makeDiag[Length[matr]], 
            Min[matr + makeDiag[Length[matr]]]]\)[\([1]\)], 
        Min[matr + makeDiag[Length[matr]]]}\)], "Input"],

Cell[TextData[{
  "Wie wir gesehen haben, werden im Verlaufe der Iteration Taxa zu gr\
\[ODoubleDot]\[SZ]eren Gruppen (Clustern) zusammengefasst. Es ist daher \
erforderlich, auch die aktuellen Cluster mitzuprotokollieren. Dies geschieht \
mit der Variablen ",
  StyleBox["clusterVec1",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  ". Zuerst initialisieren wir diese Variable."
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[CellGroupData[{

Cell[BoxData[
    \(clusterVec1 = Table[1, {Length[distMatrix]}]\)], "Input"],

Cell[BoxData[
    \({1, 1, 1}\)], "Output"]
}, Open  ]],

Cell["\<\
Ein Update dieses Clustervektors ist sehr einfach. Wir schreiben \
die Summe der Eintr\[ADoubleDot]ge in die erste der beiden Koordinaten und l\
\[ODoubleDot]schen die zweite der beiden Koordinaten.\
\>", "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[BoxData[
    \(modifyClusterVector[clusters_, coord_] := 
      Module[{tmp1}, \[IndentingNewLine]tmp1 = 
          clusters; \[IndentingNewLine]tmp1[\([coord[\([1]\)]]\)] = 
          tmp1[\([coord[\([1]\)]]\)] + 
            tmp1[\([coord[\([2]\)]]\)]; \[IndentingNewLine]tmp1 = 
          Drop[tmp1, {coord[\([2]\)]}]; \[IndentingNewLine]tmp1]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(modifyClusterVector[clusterVec1, merge1[\([1]\)]]\)], "Input"],

Cell[BoxData[
    \({1, 2}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Als N\[ADoubleDot]chstes wollen wir den graduellen, durch Zusammenfassen \
von Taxa zu Clustern gesteuerten Abbau der Distanzmatrix implementieren, so \
wie wir ihn im Verlaufe dieses Kapitels kennen gelernt haben. Die \
entsprechende Transformation der Distanzmatrix enth\[ADoubleDot]lt die \
zentrale Gleichung des UPGMA-Verfahrens, n\[ADoubleDot]mlich die \
Abstandsberechnung zwischen Clustern. Mit der (etwas unhandlichen aber \
transparenten) Befehlskombination ",
  StyleBox["Transpose[Drop[Transpose[Drop[matrix, {number}]], {number}]]",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  " l\[ODoubleDot]schen wir zun\[ADoubleDot]chst ein Taxon (als Zeile und \
Spalte) aus der Matrix. Dann berechnen wir \[UDoubleDot]ber die \
Abstandsgleichung die neuen Abst\[ADoubleDot]nde, l\[ODoubleDot]schen auch \
dort das entsprechende Taxon und setzen diesen neuen Abstandsvektor in der \
Matrix anstelle des anderen zu entfernenden Taxons wieder ein."
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[BoxData[
    \(computeNewMatrix[matr_, clusters_, coord_] := 
      Module[{tmp1, tmp2, tmp3, tmp4, tmp4b, 
          tmp5}, \[IndentingNewLine]tmp1 = \(Dimensions[
              matr]\)[\([1]\)]; \[IndentingNewLine]tmp2 = 
          Sort[coord]; \[IndentingNewLine]tmp3 = 
          Transpose[
            Drop[Transpose[
                Drop[matr, {tmp2[\([2]\)]}]], {tmp2[\([2]\)]}]]; \
\[IndentingNewLine]tmp4 = 
          Table[\((matr[\([coord[\([1]\)], k]\)]*
                    clusters[\([coord[\([1]\)]]\)] + 
                  matr[\([coord[\([2]\)], k]\)]*
                    clusters[\([coord[\([2]\)]]\)])\)/\((clusters[\([coord[\([\
1]\)]]\)] + clusters[\([coord[\([2]\)]]\)])\), {k, 1, 
              tmp1}]; \[IndentingNewLine]tmp4b = 
          Insert[Drop[Drop[tmp4, {tmp2[\([2]\)]}], {tmp2[\([1]\)]}], 0.0, 
            tmp2[\([1]\)]]; \[IndentingNewLine]tmp5 = 
          Transpose[
            Transpose[
                tmp3 /. tmp3[\([tmp2[\([1]\)]]\)] \[Rule] 
                    tmp4b] /. \(Transpose[
                    tmp3 /. tmp3[\([tmp2[\([1]\)]]\)] \[Rule] 
                        tmp4b]\)[\([tmp2[\([1]\)]]\)] \[Rule] 
                tmp4b]; \[IndentingNewLine]tmp5]\)], "Input"],

Cell[TextData[{
  "Diesen Befehl, der die neue, um ein Cluster reduzierte Matrix berechnet, k\
\[ODoubleDot]nnen wir nun auf die aktuellen Zust\[ADoubleDot]nde der \
Abstandsmatrix, des Clustervektors und der Variablen ",
  StyleBox["merge1",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  " anwenden:"
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13],

Cell[CellGroupData[{

Cell[BoxData[
    \(computeNewMatrix[distMatrix, clusterVec1, merge1[\([1]\)]] // 
      MatrixForm\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0", "1.025`"},
            {"1.025`", "0.`"}
            },
          RowSpacings->1,
          ColumnSpacings->1,
          ColumnAlignments->{Left}], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Diesen Schritt zu einer reduzierten Distanzmatrix kann man f\[UDoubleDot]r \
gr\[ODoubleDot]\[SZ]ere Ausgangsmatrizen iterieren. Die zusammengefassten \
Variablen ",
  StyleBox["merge1",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"],
  " aus jedem Iterationsschritt entsprechen im Wesentlichen der Newick-Repr\
\[ADoubleDot]sentation eines solchen Baums. \n"
}], "Text",
  FontFamily->"Times New Roman",
  FontSize->13]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 941}},
WindowSize->{507, 678},
WindowMargins->{{-19, Automatic}, {Automatic, 15}},
Magnification->1
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 553, 21, 115, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2332, 76, 196, 6, 97, "Subtitle"],
Cell[2531, 84, 1245, 21, 337, "Text"],
Cell[3779, 107, 113, 2, 50, "Input"],
Cell[3895, 111, 285, 9, 53, "Text"],

Cell[CellGroupData[{
Cell[4205, 124, 57, 1, 30, "Input"],
Cell[4265, 127, 379, 11, 71, "Output"]
}, Open  ]],
Cell[4659, 141, 1609, 42, 305, "Text"],
Cell[6271, 185, 109, 2, 30, "Input"],

Cell[CellGroupData[{
Cell[6405, 191, 68, 1, 30, "Input"],
Cell[6476, 194, 73, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6586, 200, 154, 3, 30, "Input"],
Cell[6743, 205, 49, 1, 29, "Output"]
}, Open  ]],
Cell[6807, 209, 380, 11, 34, "Text"],
Cell[7190, 222, 210, 4, 30, "Input"],
Cell[7403, 228, 469, 12, 53, "Text"],

Cell[CellGroupData[{
Cell[7897, 244, 77, 1, 30, "Input"],
Cell[7977, 247, 43, 1, 29, "Output"]
}, Open  ]],
Cell[8035, 251, 271, 6, 33, "Text"],
Cell[8309, 259, 364, 6, 110, "Input"],

Cell[CellGroupData[{
Cell[8698, 269, 82, 1, 30, "Input"],
Cell[8783, 272, 40, 1, 29, "Output"]
}, Open  ]],
Cell[8838, 276, 1059, 19, 110, "Text"],
Cell[9900, 297, 1223, 24, 190, "Input"],
Cell[11126, 323, 392, 11, 53, "Text"],

Cell[CellGroupData[{
Cell[11543, 338, 112, 2, 30, "Input"],
Cell[11658, 342, 330, 10, 55, "Output"]
}, Open  ]],
Cell[12003, 355, 465, 12, 72, "Text"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

